import time

from .base import *

class Archive:
    def __init__(self, author: NSID = '0', target: NSID = '0'):
        """
        Classe de référence pour toutes les archives.

        ## Attributs de base
        - date: `int`\n
            Date (timestamp) de l'exécution de l'archive
        - id: `NSID`\n
            Clé d'identification des archives (basée sur la date)
        - author: `NSID`\n
            ID de l'auteur de l'action
        - target: `NSID`:
            ID de la cible de l'action
        - action: `str`:\n
            Action effectuée
        - details: `dict`\n
            Ensemble de détails que les différents bots peuvent utiliser
        """
        self.date: int = round(time.time())

        self.id: NSID = NSID(self.date)
        self.author: NSID = NSID(author)
        self.target: NSID = NSID(target)

        self.action: str = ""
        self.details: dict = {
            "reason": None
        }


# Entities

class Sanction(Archive):
    def __init__(self, author: NSID, target: NSID) -> None:
        super().__init__(author, target)

        self.details: dict = {
            "reason": None,
            "major": False, # Sanction majeure ou non
            "duration": 0 # Durée en secondes , 0 = définitif
        }

class Report(Archive):
    def __init__(self, author: NSID, target: NSID) -> None:
        super().__init__(author, target)

        self.details: dict = {
            "reason": None,
            "elements": [] # Liste des pièces jointes
        }


# Community

class Election(Archive):
    def __init__(self, author: NSID, target: NSID, position: str) -> None:
        super().__init__(author, target)

        self.details = {
            "position": position,
            "positive_votes": 0,
            "total_votes": 0
        }

class Promotion(Archive):
    def __init__(self, author: NSID, target: NSID, position: str) -> None:
        super().__init__(author, target)

        self.details = {
            "position": position
        }

class Demotion(Archive):
    def __init__(self, author: NSID, target: NSID) -> None:
        super().__init__(author, target)


# Bank

class Transaction(Archive):
    def __init__(self, author: NSID, target: NSID) -> None:
        super().__init__(author, target)

        self.details = {
            "amount": 0,
            "currency": "HC",
            "reason": None
        }


import requests
import time
import urllib

from .base import NSID

default_headers = {}

class BankAccount:
    """
    Compte en banque d'une entité, individuelle ou collective.

    ## Attributs
    - id: `NSID`\n
        Identifiant du compte
    - owner: `NSID`\n
        Identifiant du titulaire du compte
    - amount: `int`\n
        Somme d'argent totale sur le compte
    - frozen: `bool`\n
        État gelé ou non du compte
    - bank: `NSID`\n
        Identifiant de la banque qui détient le compte
    - income: `int`\n
        Somme entrante sur le compte depuis la dernière réinitialisation (tous les ~ 28 jours)
    """

    def __init__(self, owner_id: NSID) -> None:
        self._url: str = ""

        self.id: NSID = NSID(owner_id)
        self.owner_id: NSID = NSID(owner_id)
        self.register_date: int = round(time.time())
        self.tag: str = "inconnu"
        self.bank: str = "HexaBank"

        self.amount: int = 0
        self.income: int = 0

        self.frozen: bool = False
        self.flagged: bool = False

    def _load(self, _data: dict):
        self.owner_id = NSID(_data['owner_id'])
        self.register_date = _data['register_date']
        self.tag = _data['tag']
        self.bank = _data['bank']

        self.amount = _data['amount']
        self.income = _data['income']

        self.frozen = _data['frozen']
        self.flagged = _data['flagged']

    def freeze(self, frozen: bool = True, reason: str = None) -> None:
        res = requests.post(f"{self._url}/freeze?frozen={str(frozen).lower()}", headers = default_headers, json = {
            "reason": reason
        })

        if res.status_code == 200:
            self.frozen = frozen
        else:
            print(res.text)
            res.raise_for_status()

    def flag(self, flagged: bool = True, reason: str = None) -> None:
        res = requests.post(f"{self._url}/flag?flagged={str(flagged).lower()}", headers = default_headers, json = {
            "reason": reason
        })

        if res.status_code == 200:
            self.flagged = flagged
        else:
            res.raise_for_status()

    def debit(self, amount: int, reason: str = None, target: NSID = None, loan: NSID = None, digicode: str = None) -> None:
        _target_query = f"&target={target}"
        _loan_query = f"&loan_id={loan}"

        res = requests.post(f"{self._url}/debit?amount={amount}{_target_query if target else ''}{_loan_query if loan else ''}", headers = default_headers, json = {
            "reason": reason,
            "digicode": digicode
        })

        if res.status_code == 200:
            self.amount -= amount
        else:
            res.raise_for_status()

    def deposit(self, amount: int, reason: str = None) -> None:
        res = requests.post(f"{self._url}/deposit?amount={amount}", headers = default_headers, json = {
            "reason": reason,
        })

        if res.status_code == 200:
            self.amount -= amount
        else:
            res.raise_for_status()

class Item:
    """
    Article d'inventaire qui peut circuler sur le serveur

    ## Attributs
    - id: `NSID`\n
        Identifiant de l'objet
    - name: `str`\n
        Nom de l'objet
    - emoji: `str`\n
        Emoji lié à l'objet
    """

    def __init__(self) -> None:
        self.id: NSID = NSID(round(time.time()))
        self.name: str = "Unknown Object"
        self.emoji: str = ":light_bulb:"
        self.category: str = "common"
        self.craft: dict = {}

    def _load(self, _data: dict):
        self.name = _data['name']
        self.emoji = _data['emoji']
        self.category = _data['category']
        self.craft = _data['craft']

    def rename(self, new_name: str):
        res = requests.post(f"{self._url}/rename?name={new_name}", headers = default_headers)

        if res.status_code == 200:
            self.name = new_name
        else:
            res.raise_for_status()

class Sale:
    """
    Vente mettant en jeu un objet

    ## Attributs
    - id: `NSID`\n
        Identifiant de la vente
    - item: `NSID`\n
        Identifiant de l'objet mis en vente
    - quantity: `int`\n
        Quantité d'objets mis en vente
    - price: `int`\n
        Prix total du lot
    - seller_id: `NSID`\n
        Identifiant du vendeur
    """

    def __init__(self, item: Item) -> None:
        self._url: str = ""

        self.id: NSID = NSID(round(time.time()))
        self.open: bool = True
        self.seller_id: NSID = NSID('0')

        self.item_id: NSID = NSID(item.id)
        self.quantity: int = 1
        self.price: int = 0

    def _load(self, _data: dict):
        self.id = _data['json']
        self.open = _data['open']
        self.seller_id = NSID(_data['seller_id'])

        self.item_id = NSID(_data['item_id'])
        self.quantity = _data['quantity']
        self.price = _data['price']

class Inventory:
    """
    Inventaire d'un membre

    ## Attributs
    - id: `NSID`\n
        ID de l'inventaire
    - owner_id: `NSID`\n
        ID du propriétaire de l'inventaire
    - tag: `str`\n
        Étiquette de l'inventaire
    - register_date: `int`\n
        Date (timestamp) de création de l'inventaire
    - items: `dict[NSID, int]`\n
        Collection d'objets et leur quantité
    """

    def __init__(self, owner_id: NSID) -> None:
        self._url: str = ""

        self.id: NSID = NSID(owner_id)
        self.owner_id: NSID = NSID(owner_id)

        self.tag: str = "inconnu"
        self.register_date: int = 0

        self.items: dict[NSID, int] = {}

    def _load(self, _data: dict):
        self.id = NSID(_data['id'])
        self.owner_id = NSID(_data['owner_id'])

        self.tag = _data['tag']
        self.register_date = _data['register_date']

        self.items = _data['items']

    def deposit_item(self, item: Item, giver: NSID = None, quantity: int = 1, digicode: str = None):
        res = requests.post(f"{self._url}/deposit?item={item.id}&amount={quantity}", headers = default_headers, json = {
            "giver": giver,
            "digicode": digicode
        })

        if res.status_code == 200:
            if self.objects[item.id] > quantity:
                self.objects[item.id] -= quantity
            else:
                self.objects[item.id] = 0
        else:
            res.raise_for_status()

    def sell_item(self, item: Item, price: int, quantity: int = 1, digicode: str = None) -> NSID:
        res = requests.post(f"{self._url}/sell_item?item={item.id}&quantity={quantity}&price={price}", headers = default_headers, json = {
            "digicode": digicode
        })

        if res.status_code == 200:
            return NSID(res.json()['sale_id'])
        else:
            res.raise_for_status()


import time

from ..cls.base import *
from ..cls.archives import *
from ..cls.economy import *

from ..cls import economy # Pour les default_headers

class EconomyInstance(Instance):
    """Indisponible dans cette version."""

    def __init__(self, url: str, token: str) -> None:
        super().__init__(url, token)

        economy.default_headers = self.default_headers

    """
    ---- COMPTES EN BANQUE ----
    """

    def get_account(self, id: NSID) -> BankAccount:
        """
        Récupère les informations d'un compte bancaire.

        ## Paramètres
        id: `NSID`\n
            ID du compte.

        ## Renvoie
        - `.BankAccount`
        """

        id = NSID(id)
        res = requests.get(f"{self.url}/bank/accounts/{id}", headers = self.default_headers)

        if res.status_code == 200:
            _data = res.json()
        else:
            res.raise_for_status()
            return

        if _data is None:
            return None

        account = BankAccount(id)
        account._url = f"{self.url}/bank/accounts/{account.id}"

        account._load(_data)

        return account

    def save_account(self, account: BankAccount) -> str:
        """
        Sauvegarde un compte bancaire dans la base de données.

        ## Paramètres
        - account: `.BankAccount`\n
            Compte à sauvegarder
        """

        _data = {
            'id': NSID(account.id),
            'amount': account.amount,
            'frozen': account.frozen, 
            'owner_id': account.owner_id, 
            'bank': account.bank,
            'income': account.income
        }

        res = requests.put(f"{self.url}/bank/register_account?owner={_data['owner_id']}", headers = self.default_headers, json = _data)

        if res.status_code == 200:
            account._url = f"{self.url}/bank/accounts/{account.id}"
            account.id = res.json()['id']

            return res.json()['digicode']
        else:
            res.raise_for_status()

    def fetch_accounts(self, **query: typing.Any) -> list[BankAccount]:
        """
        Récupère une liste de comptes en banque en fonction d'une requête.

        ## Paramètres
        query: `**dict`\n
            La requête pour filtrer les comptes.

        ## Renvoie
        - `list[.BankAccount]`
        """

        query = "&".join(f"{k}={ urllib.parse.quote(v) }" for k, v in query.items())

        _res = requests.get(f"{self.url}/fetch/accounts?{query}", headers = self.default_headers)

        if _res.status_code == 200:
            _data = _res.json()
        else:
            _res.raise_for_status()
            return []

        res = []

        for _acc in _data:
            if not _acc: continue

            account = BankAccount(_acc["owner_id"])

            account.id = NSID(_acc['id'])
            account._url = f"{self.url}/bank/accounts/{account.id}"

            account._load(_acc)

            res.append(account)

        return res

    """
    ---- INVENTAIRES ----
    """

    def get_inventory(self, id: NSID) -> Inventory:
        """
        Récupère les informations d'un inventaire.

        ## Paramètres
        id: `NSID`\n
            ID de l'inventaire.

        ## Renvoie
        - `.Inventory`
        """

        id = NSID(id)
        res = requests.get(f"{self.url}/bank/inventories/{id}", headers = self.default_headers)

        if res.status_code == 200:
            _data = res.json()
        else:
            res.raise_for_status()
            return

        if _data is None:
            return None

        inventory = Inventory(id)
        inventory._url = f"{self.url}/bank/inventories/{inventory.id}"

        inventory._load(_data)

        return inventory

    def save_inventory(self, inventory: Inventory) -> str:
        """
        Sauvegarde un inventaire dans la base de données.

        ## Paramètres
        - inventory: `.Inventory`\n
            Inventaire à sauvegarder
        """

        _data = inventory.__dict__

        res = requests.put(f"{self.url}/bank/register_inventory?owner={_data['owner_id']}", headers = self.default_headers, json = _data)

        if res.status_code == 200:
            inventory._url = f"{self.url}/bank/inventories/{inventory.id}"
            inventory.id = res.json()['id']

            return res.json()['digicode']
        else:
            res.raise_for_status()

    def fetch_inventories(self, **query: typing.Any) -> list[Inventory]:
        """
        Récupère une liste d'inventaires en fonction d'une requête.

        ## Paramètres
        query: `**dict`\n
            La requête pour filtrer les inventaires.

        ## Renvoie
        - `list[.Inventory]`
        """

        query = "&".join(f"{k}={ urllib.parse.quote(v) }" for k, v in query.items())

        _res = requests.get(f"{self.url}/fetch/inventories?{query}", headers = self.default_headers)

        if _res.status_code == 200:
            _data = _res.json()
        else:
            _res.raise_for_status()
            return []

        res = []

        for _inv in _data:
            if not _inv: continue

            inventory = Inventory(_inv["owner_id"])

            inventory.id = NSID(_inv['id'])
            inventory._url = f"{self.url}/bank/inventories/{inventory.id}"

            inventory._load(_inv)

            res.append(inventory)

        return res

    """
    ---- ITEMS ----
    """

    def get_item(self, id: NSID) -> Item:
        """
        Récupère les informations d'un item.

        ## Paramètres
        id: `NSID`\n
            ID de l'item.

        ## Renvoie
        - `.Item`
        """

        id = NSID(id)
        res = requests.get(f"{self.url}/marketplace/items/{id}", headers = self.default_headers)

        if res.status_code == 200:
            _data = res.json()
        else:
            res.raise_for_status()
            return

        if _data is None:
            return None

        item = Item()
        item.id = id
        item._url = f"{self.url}/marketplace/items/{item.id}"

        item._load(_data)

        return item

    def save_item(self, item: Item) -> None:
        """
        Sauvegarde un item dans le marketplace.

        ## Paramètres
        - item: `.Item`\n
            Item à sauvegarder
        """

        _data = item.__dict__

        res = requests.put(f"{self.url}/marketplace/register_item", headers = self.default_headers, json = _data)

        if res.status_code == 200:
            item._url = f"{self.url}/bank/inventories/{item.id}"
            item.id = res.json()['id']
        else:
            res.raise_for_status()

    def fetch_items(self, **query: typing.Any) -> list[Item]:
        """
        Récupère une liste d'items en fonction d'une requête.

        ## Paramètres
        query: `**dict`\n
            La requête pour filtrer les items.

        ## Renvoie
        - `list[.Item]`
        """

        query = "&".join(f"{k}={ urllib.parse.quote(v) }" for k, v in query.items())

        _res = requests.get(f"{self.url}/fetch/items?{query}", headers = self.default_headers)

        if _res.status_code == 200:
            _data = _res.json()
        else:
            _res.raise_for_status()
            return []

        res = []

        for _item in _data:
            if not _item: continue

            item = Item()

            item.id = NSID(_item['id'])
            item._url = f"{self.url}/marketplace/items/{item.id}"

            item._load(_item)

            res.append(item)

        return res


    """
    ---- VENTES ----
    """

    def get_sale(self, id: NSID) -> Sale:
        """
        Récupère les informations d'une annonce.

        ## Paramètres
        id: `NSID`\n
            ID de la annonce.

        ## Renvoie
        - `.Sale`
        """

        id = NSID(id)
        res = requests.get(f"{self.url}/marketplace/sales/{id}", headers = self.default_headers)

        if res.status_code == 200:
            _data = res.json()
        else:
            res.raise_for_status()
            return

        if _data is None:
            return None

        sale = Sale()
        sale.id = id
        sale._url = f"{self.url}/marketplace/sales/{sale.id}"

        sale._load(_data)

        return sale

    def fetch_sales(self, **query: typing.Any) -> list[Sale]:
        """
        Récupère une liste d'annonces en fonction d'une requête.

        ## Paramètres
        query: `**dict`\n
            La requête pour filtrer les annonces.

        ## Renvoie
        - `list[.Sale]`
        """

        query = "&".join(f"{k}={ urllib.parse.quote(v) }" for k, v in query.items())

        _res = requests.get(f"{self.url}/fetch/sales?{query}", headers = self.default_headers)

        if _res.status_code == 200:
            _data = _res.json()
        else:
            _res.raise_for_status()
            return []

        res = []

        for _sale in _data:
            if not _sale: continue

            sale = Sale()

            sale.id = NSID(_sale['id'])
            sale._url = f"{self.url}/marketplace/sales/{sale.id}"

            sale._load(_sale)

            res.append(sale)

        return res


import io
import math
import os
from PIL import Image

def open_asset(path: str) -> bytes:
    curr_dir = os.path.dirname(os.path.abspath(os.path.join(__file__)))
    asset_path = os.path.join(curr_dir, 'assets', path)

    image = Image.open(asset_path)
    val = io.BytesIO()

    image.save(val, format = 'PNG')

    return val.getvalue()

def compress_image(data: bytes, _max: int = 1000 ** 2) -> bytes:
    img = Image.open(io.BytesIO(data))
    size = 2 * ( math.floor(math.sqrt(_max),) )

    img.resize(size)

    val = io.BytesIO()
    img.save(val)

    return val.getvalue()

def warn(prompt: str):
    print("\033[1;33mWarning:\033[0m", prompt)